@@ -335,7 +335,7 @@ def _auto_remote(path_to_tx, url):
             if r.has_key('project_slug'):
                 resources.append(r['project_slug'])
             else:
-                resources.append('.'.join(vars['project'], r['slug']))           
+                resources.append('.'.join(vars['project'], r['slug']))
         utils.MSG("%s resources found. Configuring..." % len(resources))
     elif type == 'resource':
         utils.MSG("Getting details for resource %s" % vars['resource'])
@@ -401,7 +401,7 @@ def cmd_push(argv, path_to_tx):
     force_creation = options.force_creation
 
     if options.languages:
-        languages = options.languages.split(',') 
+        languages = options.languages.split(',')
     else:
         languages = []
 
@@ -409,7 +409,7 @@ def cmd_push(argv, path_to_tx):
         resources = options.resources.split(',')
     else:
         resources = []
-    
+
     skip = options.skip_errors
 
     # instantiate the project.Project
@@ -471,7 +471,7 @@ def cmd_pull(argv, path_to_tx):
             " -a|--all option")
 
     if options.languages:
-        languages = options.languages.split(',') 
+        languages = options.languages.split(',')
     else:
         languages = []
 
@@ -611,13 +611,17 @@ def cmd_status(argv, path_to_tx):
         utils.MSG("Translation Files:")
         slang = prj.get_resource_option(res, 'source_lang')
         sfile = prj.get_resource_option(res, 'source_file') or "N/A"
+        lang_map = prj.get_resource_lang_mapping(res)
         utils.MSG(" - %s: %s (%s)" % (utils.color_text(slang, "RED"),
             sfile, utils.color_text("source", "YELLOW")))
         files = prj.get_resource_files(res)
         fkeys = files.keys()
         fkeys.sort()
         for lang in fkeys:
-            utils.MSG(" - %s: %s" % (utils.color_text(lang, "RED"),
+            local_lang = lang
+            if lang in lang_map.values():
+                local_lang = lang_map.flip[lang]
+            utils.MSG(" - %s: %s" % (utils.color_text(local_lang, "RED"),
                 files[lang]))
 
         utils.MSG("")
